/* Automat je reprezentovn polem pol, kde kad takov subpole
 * je stav, jeho prvky jsou jednotliv pechody 
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <locale.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "vacek.h"
#include "cross.h"
#include "automat.h"

/* --------------------------- DEFINE SEKCE --------------------------------- */

#define TRUE	1
#define FALSE	0

LOCAL  unsigned char pom_retezce[MAX_DELKA_RETEZCE+1]; /* etzce pro vpis */
GLOBAL Prechod automat[MAX_POCET_PRECHODU];     /* pole pechod */
GLOBAL Stav koren;				/* automat[0].cis_hodnota */
GLOBAL unsigned int velikost_automatu;		/* celkov poet pechod */
GLOBAL unsigned int pocatecni_stav;		/* jeho pozice v automatu */

/* -------------------------------------------------------------------------- */
  	        /* pln funkn prototypy globlnch funkc */
/* -------------------------------------------------------------------------- */

GLOBAL void nacti_automat(char *cesta);
GLOBAL void uloz_automat(void);
GLOBAL int testuj_retezec(unsigned char *retezec);
GLOBAL void vypis_retezce(unsigned int pozice, int ret_poz, FILE *soubor);
GLOBAL unsigned int traverzuj(unsigned int n, Slovo *w, int kolik);
GLOBAL boolean je_slovo(unsigned int start, Slovo *w);
/* -------------------------------------------------------------------------- */
  	        /* pln funkn prototypy loklnch funkc */
/* -------------------------------------------------------------------------- */

LOCAL void vypis_chybu(char *hlaska);

/* -------------------------------------------------------------------------- */
			    /* Procedury a funkce */
/* -------------------------------------------------------------------------- */

/* Pi chyb zahla o co jde a skoni */
LOCAL void vypis_chybu(char *hlaska)
{
        fprintf(stderr, "%s\n", hlaska);
	fprintf(stderr, "--konme...\n");
	exit(EXIT_FAILURE);
}

/* -------------------------------------------------------------------------- */


/* nacti automat ze vstupnho souboru */
GLOBAL void nacti_automat(char *cesta)
{

    FILE *automat_soubor;		/* vstupn soubor s automatem */


    
    if ((automat_soubor = fopen(cesta, "rb")) == NULL)
	vypis_chybu("Nemohu otevt vstupn soubor s automatem!");

    printf("Nahrvm soubor se slovnkem '%s'\n", cesta);

    velikost_automatu = fread(automat, sizeof(automat[0]), 
	                      MAX_POCET_PRECHODU, automat_soubor);

    fclose(automat_soubor);

    
    if (velikost_automatu >= 2) {
        /* vytvo pomocn stav ukazujc na poten stav */
	memcpy(&pocatecni_stav, &automat[0], sizeof(automat[0]));
	automat[0].t.do_stavu = automat[0].cis_hodnota;
	
	/* Koen je pechod, jeho poloka do_stavu ukazuje na index
	 * v poli automat, kde zan prvn pechod. Poloka 'znak' je
	 * nedefinovan.
	*/
	koren = automat[0].cis_hodnota;

	if (pocatecni_stav < velikost_automatu)
	    return;	/* OK, vstup naten spn */
	
    }/* end if */
    
    vypis_chybu("Chyba v souboru s automatem!");
    
}/* end nacti_automat() */
 

/* -------------------------------------------------------------------------- */

/* Testuj pslunost etzce v automatu
 * Vrac: 0 - etzec neptomen
 *        1 - etzec nalezen
*/
GLOBAL int testuj_retezec(unsigned char *retezec)
{
    unsigned int pozice = 0;
    unsigned short int i;
    unsigned char w;

    for (i = 0; retezec[i]; i++) {
	
	/* zskej pozici nsledujcho stavu */
        pozice = automat[pozice].t.do_stavu;
        w = retezec[i];

        if (pozice > velikost_automatu)
	    vypis_chybu("Chyba v automatu!");
	
        /* nalezni pechod pod pslunm znakem */
        while (automat[pozice].t.znak != (unsigned int) w) {
	    
	    if (automat[pozice++].t.posledni)
		return 0;		/* z danho stavu nevede pechod */
	    
                if (pozice > velikost_automatu)
		    vypis_chybu("Chyba v automatu!");
		
        }/* end while */
	
    }/* end for */
    
    return(automat[pozice].t.terminal); /* nalezen pechod je koncov,
					   etzec je ptomen */

}/* end testuj_retezec() */



/* -------------------------------------------------------------------------- */

/* Vypi vechny etzce obsaen v automatu na pozici v automatu*/
GLOBAL void vypis_retezce(unsigned int pozice, int ret_poz, FILE *soubor)
{
    int i;
    
    if (pozice == 0)
	return;

    if (pozice > velikost_automatu)
	vypis_chybu("Chyba v automatu!");
   

    do {
    
	pom_retezce[ret_poz] = (unsigned char) (automat[pozice].t.znak);

	if (automat[pozice].t.terminal) {
	
	    /* vypis retezec, pokud aktuln znak pechodu je koncov */
	    for (i = 0; i <= ret_poz; i++)
		putc(pom_retezce[i], soubor); /* zapisuj do souboru */
	
	    putc('\n', soubor);
	
	}/* end if */
	
	/* a prove rekurzivn pro vechny znaky v aktulnm stavu */
        vypis_retezce(automat[pozice].t.do_stavu, ret_poz+1, soubor);
	
    } while (!(automat[pozice++].t.posledni));
    
}/* end vypis_retezce() */


/* -------------------------------------------------------------------------- */

/* ulo automat v textov podob do souboru */
GLOBAL void uloz_automat(void) {
    
    FILE *automat_vypis;	/* vstupn soubor s obsahem automatu */

    if ((automat_vypis = fopen("./automat.vypis", "w")) == NULL)
	vypis_chybu("Nemohu otevt soubor pro zpis!");

    if (setvbuf(automat_vypis, NULL, _IOFBF, 8192) != 0)
	vypis_chybu("Chyba pi nastaven blokovho bufferovn!");
	

    vypis_retezce(pocatecni_stav, 0, automat_vypis);

    fclose(automat_vypis);

}
/* -------------------------------------------------------------------------- */

/* Projde ze stavu 'Stav' 'kolik' znak ze slova 'Slovo' a vrt
 * stav, do kterho se dostal.
*/
GLOBAL Stav traverzuj(Stav n, Slovo *w, int kolik) {
        
    Prechod p;
    
    if (n>velikost_automatu) {
	fprintf(stderr, "POZOR! STAV MIMO MEZE! (automat.c/traverzuj)\n");
	return(pocatecni_stav);
    }

    if(kolik == w->n)	/* Proli jsme cel slovo 'w' */
	return(n);
    
    if(n==0)
	return(0);
    
    do {

	p.t = automat[n++].t;
	
	    if( p.t.znak == (unsigned int) w->c[kolik]) {
                return(traverzuj(p.t.do_stavu, w, kolik+1));
            }
	} while(!p.t.posledni);
    
    return(0); /* Nenalezena dn shoda */

}/* end traverzuj() */

/* -------------------------------------------------------------------------- */

/* Projde ze stavu 'start' po znacch z 'w' a otestuje, zda
 * jsme doli do koncovho stavu (=slovo je platn).
 * Vrac:  0 - 'w' nen platn
 *         1 - 'w' je platn
*/

GLOBAL boolean je_slovo(Stav start, Slovo *w) {
    
    Prechod	e;
    Stav 	n;
    boolean 	nalezeno;
    int		j;

    n = start;
    if (n>velikost_automatu) {
	fprintf(stderr, "POZOR! STAV MIMO MEZE! (automat.c/je_slovo)\n");
	return(pocatecni_stav);
    }

    for (j=0; j<w->n; j++) {
	nalezeno = FALSE;
	
        do {
	    e.t = automat[n++].t;
	    
		/* Shoduje se znak v pechodu s 'w'? */
		if(e.t.znak == (unsigned int) w->c[j]) {
		    nalezeno = TRUE;
		    break;              /* Pokrauj ve FORu */
                }
        } while (!e.t.posledni);        /* Posledn pechod stavu? */

	if(!nalezeno)			/* Nenali jsme pechod pod znakem */
	    return(FALSE);

	n = e.t.do_stavu;		/* Nali, pokrauj v hledn dle */

    }/* end for */
	
    if(e.t.terminal)			/* Je koncov? */
	return(TRUE);
    else
	return(FALSE);
	
}/* end je_slovo() */
 
/* -------------------------------------------------------------------------- */

/* END_OF_FILE automat.c */
