#include<stdio.h>
#include<string.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "cross.h"
#include "skore.h"
#include "automat.h"
#include "vacek.h"
#include "deska.h"
#include "prihradka.h"
#include "tahy_gui.h"

#define TRUE                            1
#define FALSE                           0

/* --------------------- DEFINICE GLOBALNCH PROMNNCH --------------------- */

/* Struktura pro kontrolu vertiklnch slov */
/* Prvn dimenze udv, jak je natoen desky */
GLOBAL vcross cross[MAX_ROZMER][MAX_ROZMER];

/* --------------------- DEFINICE LOCALNCH PROMNNCH ---------------------- */

/* --------------- PLN FUNKN PROTOTYPY GLOBALNCH FUNKC ---------------- */

GLOBAL void cross_resetuj_struktury(void);
GLOBAL void aktualizuj_cross_set(int horiz);
GLOBAL boolean je_v_cross_set(tpozice p, torientace o, char pismeno);
GLOBAL boolean JEPISMENO(tpozice p);

/* ---------------- PLN FUNKN PROTOTYPY LOCALNCH FUNKC ---------------- */

LOCAL void napln_cross_set(tpozice p, torientace o);
LOCAL void napln_prazdna_pole(tpozice p, torientace o);
LOCAL void cross_pridej(tpozice p, torientace o, char pismeno);
#ifdef CROSS_DEBUG
LOCAL void cross_vypis(tpozice p, torientace o);
#endif

/* -------------------------------------------------------------------------- */
/* ------------------------ DEFINICE GLOBALNCH FUNKC ---------------------- */
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */

GLOBAL boolean JEPISMENO(tpozice p) {

    if (OKRAJ(p))
	return(FALSE);

    if (board_is_field_vacant(p.x-1, p.y-1) == TRUE)
	return(FALSE);

    return(TRUE);
    
}

/* -------------------------------------------------------------------------- */
/* ------------------------ DEFINICE LOCALNCH FUNKC ----------------------- */
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */

/*
 * Vlo psmeno do crosscheck mnoiny, opakovan vskyt
 * nevad, vrac se struktura tcross s nastavenmi hodnotami
*/
LOCAL void cross_pridej(tpozice p, torientace o, char pismeno) {
  
    tcross cross_set; 
    unsigned int i, blowhi=FALSE;
    unsigned long maska=0;

    cross_set.hi = (cross[p.x][p.y])[o].hi;
    cross_set.lo = (cross[p.x][p.y])[o].lo;
    
    for(i=0; i<MAX_VEL_ABECEDY-1; i++)
	if (ABECEDA[i] == pismeno) break;
   
    if (i >= S_BYTE*S_ULONG) {
	i -= S_BYTE*S_ULONG;
	blowhi = TRUE;
    }	
        
    maska = 1 << i;

    if (blowhi) cross_set.hi |= maska;
    else	cross_set.lo |= maska;

    /* Vra zpt na sv msto */  
    (cross[p.x][p.y])[o].hi = cross_set.hi;
    (cross[p.x][p.y])[o].lo = cross_set.lo;

}/* end pridej_do_cross_set() */

/* -------------------------------------------------------------------------- */

#ifdef CROSS_DEBUG
/*
 * Ve str_cross funkce vrac poet psmen v crosscheck mnoin
 * uren hodnotami v cross_set (lo, hi). Navc jsou tato psmena
 * vrcena zpt v etzci str_cross.
*/
LOCAL void cross_vypis(tpozice p, torientace o) {

    char s[256] = "";

    int i, k=0;
    unsigned long int maska=0;

    for(i=0;i<S_BYTE*S_ULONG;i++) {
	maska = 1 << i;
	if (maska & (cross[p.x][p.y])[o].lo) {
	    s[k] = ABECEDA[i];
	    k++;
	}
    }
    for(i=0;i<S_BYTE*S_ULONG;i++) {
	maska = 1 << i;
	if (maska & (cross[p.x][p.y])[o].hi) {
	    s[k] = ABECEDA[S_BYTE*S_ULONG+i];
	    k++;
	}
    }

    /* Ukonme korektn etzec */
    s[k] = '\0';

}/* end of cross_vypis() */
#endif
/* -------------------------------------------------------------------------- */

/* Otestuje, zda je psmeno obsaeno v mnoin psmen, kter jsou pro
 * danou przdnou pozici na desce a danou orientaci pouiteln.
 * Vrac: FALSE - nepat
 *        TRUE  - jinak
*/
GLOBAL boolean je_v_cross_set(tpozice p, torientace o, char pismeno) {
    
    unsigned int i, blowhi=FALSE;
    unsigned long maska=0;

    for(i=0; i<MAX_VEL_ABECEDY-1; i++)
	if (ABECEDA[i] == pismeno) 	/* Najdi index psmene v abeced */
	    break;

    if (i == MAX_VEL_ABECEDY-1)		/* Nenali jsme toto psmeno => FALSE */
	return(FALSE);
   
    if (i >= S_BYTE*S_ULONG) {
	i -= S_BYTE*S_ULONG;
	blowhi = TRUE;
    }	
        
    maska = 1 << i;

    if (blowhi)
	return ( ((cross[p.x][p.y])[o].hi & maska) ? TRUE : FALSE);
    else
	return ( ((cross[p.x][p.y])[o].lo & maska) ? TRUE : FALSE);
	
}/* end of je_v_cross_set() */

/* -------------------------------------------------------------------------- */

/* Pro kad nalezen przdn pole sousedc s prv odehranm kamenem
 * uri psmena, kter buto jako prefix nebo suffix tvo s tmto kamenem
 * platn slova. Pklad: odehrl se kmen 'e'. Prefixem me bt nap.
 * 'n','v','s' a suffixem pak 'l','m','n' (nzvy psmen).
 * Tyto mnoiny psmen se v literatue oznauj jako "cross-set".
*/
LOCAL void napln_prazdna_pole(tpozice p, torientace o) {

    tpozice le, pr;
    Slovo lslovo, pslovo;
    Stav n;
    Prechod e;
    int dskore=0;


    lslovo.n = pslovo.n = 0;
    
    if(OKRAJ(p))		/* Jsme ji mimo desku? */
	return;

    /* Hledej slovo vlevo */
    for(le=PRED(p,o); JEPISMENO(le); le = PRED(le,o));
    le = ZA(le,o);
	
    while (JEPISMENO(le)) {
	lslovo.c[lslovo.n++] = PISMENO(le);
	dskore += HODNOTA(le);
        le = ZA(le,o);
    }
	
    /* Hledej slovo vpravo */
    for(pr=ZA(p,o); JEPISMENO(pr); pr = ZA(pr,o)) {
	pslovo.c[pslovo.n++] = PISMENO(pr);
	dskore += HODNOTA(pr);
    }
	
    /* Skre pro przdn pole je seteno z hodnot vpravo a vlevo
     * od tohoto pole.
    */
    score[p.x][p.y][o] = dskore;

    /* Nic nen povoleno, ve je na nulch */
    (cross[p.x][p.y])[o].lo = 0; (cross[p.x][p.y])[o].hi = 0;
   
    /* Projdi graf a kam dojde */ 
    n = traverzuj(pocatecni_stav, &lslovo, 0);
	
	if( n>0 )	/* Nkam jsme doli */
	    
	    do {
		/* Zkopruj pechod do 'e' */
		e.t = automat[n++].t;

                if ( (pslovo.n && je_slovo(e.t.do_stavu, &pslovo)) ||
                    (!pslovo.n && e.t.terminal) ) {
			cross_pridej(p, o, e.t.znak);
                        /* printf("%c, ", e.t.znak); */
                }

            } while (!(e.t.posledni));
	
#ifdef CROSS_DEBUG
        printf("\n");

	cross_vypis(p, o);
#endif
	
}/* end napln_prazdna_pole() */

/* -------------------------------------------------------------------------- */

/* Najdi przdn pole pro pozici 'p' vlevo/vpravo resp. nahoe/dole,
 * podle orientace pro n zavolej dal funkci.
*/
LOCAL void napln_cross_set(tpozice p, torientace o) {

    tpozice le, pr;	/* VleVO, VprAVO */

    for(pr=p; JEPISMENO(pr); pr=ZA(pr,o));
    for(le=p; JEPISMENO(le); le=PRED(le,o));
    
#ifdef CROSS_DEBUG 
    printf("vlevo=[%d,%d],vpravo=[%d,%d],x=%d,y=%d,ori=%d\n", 
    	    le.x, le.y, pr.x, pr.y, p.x, p.y, o);
#endif
    
    napln_prazdna_pole(le,o);
    napln_prazdna_pole(pr,o);

}

/* -------------------------------------------------------------------------- */

GLOBAL void cross_resetuj_struktury(void) {
    
    int i, j;
    
    /* cross je struktura (lo, hi) */
        for(i=0; i<MAX_ROZMER; i++)
	    for(j=0; j<MAX_ROZMER; j++) {
		(cross[i][j])[0].lo = ONES;
		(cross[i][j])[1].lo = ONES;
		(cross[i][j])[0].hi = ONES2;
		(cross[i][j])[1].hi = ONES2;
	    }
}

/* -------------------------------------------------------------------------- */

/* Pro kad odehran kamen a kad przdn sousedc pole
 * spoti mnoinu psmen, kter je mono na tato pole umstit.
*/
GLOBAL void aktualizuj_cross_set(int horiz) {

    int i;
    tpozice pos;		/* struktura x,y */
    torientace o;

    o = (horiz == HORIZONTAL) ? ZD : SD;

    for(i=0;i<MAX_PISMEN_PRIHRADKA;i++) {
	
	if (rack_placed_tiles_coords[i].which != EMPTY_TILE) {

	    pos.x = rack_placed_tiles_coords[i].x+1;
	    pos.y = rack_placed_tiles_coords[i].y+1;

	    napln_cross_set(pos, KOLMO(o));
	    
	}/* end if */
	
    }/* end for */
   
    napln_cross_set(pos, o); 
    
}/* end aktualizuj_cross_set() */

/* -------------------------------------------------------------------------- */

/* END_OF_FILE cross.c */

