#include<gnome.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "vacek.h"
#include "gui_okno.h"
#include "gui_menu.h"
#include "gui_deska.h"
#include "gui_ovladace.h"
#include "gui_prihradky.h"
#include "gui_cetnosti.h"
#include "gui_nastaveni.h"
#include "gui_load.h"
#include "gui_save.h"
#include "gui_zakazana.h"
#include "gui_logovani.h"
#include "gui_rekord.h"
#include "gui_help.h"
#include "gui_vyznamy.h"
#include "hra.h"

/* -------------------- DEKLARACE GLOBLNCH FUNKC ------------------------- */

GLOBAL void menu_pripoj_menu(GtkWidget *app);
GLOBAL void menu_pripoj_hinty(GtkWidget *appbar);

/* -------------------------------------------------------------------------- */
/* ---------------------- DEFINICE HLAVNHO MENU ---------------------------- */
/* -------------------------------------------------------------------------- */

/* Menu Hra */
LOCAL GnomeUIInfo game_menu[] = {

  GNOMEUIINFO_MENU_NEW_GAME_ITEM(hra_nova_hra, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM(gui_load_game_callback, NULL),
  GNOMEUIINFO_MENU_SAVE_ITEM(gui_save_game_callback, NULL),
  GNOMEUIINFO_MENU_END_GAME_ITEM(hra_ukonci_hru, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM(gtk_main_quit, NULL),
  GNOMEUIINFO_END
};

/* Menu Help */
LOCAL GnomeUIInfo help_menu[] = {
  {
    GNOME_APP_UI_ITEM, _("Ovldn"),
    _("Zobraz npovdu k ovldn programu."),
    gui_help_zobraz_ovladani, NULL,
    NULL, 0, 0, 'h', GDK_CONTROL_MASK
  },
  {
    GNOME_APP_UI_ITEM, _("Pravidla"),
    _("Zobraz pravidla Scrabble podle ASu."),
    gui_help_zobraz_pravidla, NULL,
    NULL, 0, 0, 'i', GDK_CONTROL_MASK
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
  GNOMEUIINFO_END
};


/* Menu Zobrazen */
LOCAL GnomeUIInfo view_menu[] = {

    GNOMEUIINFO_ITEM (_("Vra poloen kameny do pihrdky"),
		     _("Vrt poloen kameny z desky zpt do pihrdky"),
		     rack_return_played_tiles_home, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM ( _("Zobraz vek s psmeny"),
    _("Zobraz zbyl psmena ve vku."),
    gui_zobraz_cetnosti_zbylych, NULL),
    GNOMEUIINFO_ITEM ( _("Zobraz etnosti psmen"),
    _("Zobraz poty psmen pro hru."),
    gui_zobraz_cetnosti, NULL),
    GNOMEUIINFO_ITEM ( _("Zobraz pihrdky hr"),
    _("Zobraz psmena v pihrdkch hr."),
    gui_zobraz_prihradky, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_SCORES_ITEM(gui_rekord_zobraz, NULL),
    GNOMEUIINFO_END
};

/* Pomocn submenu pro nastaven obtnosti */
LOCAL GnomeUIInfo radio_list[] = {
  
    GNOMEUIINFO_ITEM_DATA ( N_("Zatenk"),
    N_("Nastav rove na 'zatenk'"),
    gui_ovladace_obtiznost_callback, (gpointer) 2, NULL),

    GNOMEUIINFO_ITEM_DATA (N_("Pokroil"),
    N_("Nastav rove na 'pokroil'"),
    gui_ovladace_obtiznost_callback, (gpointer) 1, NULL),

    GNOMEUIINFO_ITEM_DATA (N_("Mistr"),
    N_("Nastav rove na 'mistr'"),
    gui_ovladace_obtiznost_callback, (gpointer) 0, NULL),
    
    GNOMEUIINFO_END
};

/* Podmenu 'Obtnost' */
LOCAL GnomeUIInfo obtiznost_submenu[] = {
    GNOMEUIINFO_RADIOLIST(radio_list),
    GNOMEUIINFO_END
};

/* Menu Nstroje */
LOCAL GnomeUIInfo tools_menu[] = {
#if 0
  GNOMEUIINFO_MENU_HINT_ITEM( gui_nabidni_tip, NULL),
#endif
  {
    GNOME_APP_UI_ITEM, _("Platnost slova"),
    _("Otestuje platnost zadanho slova."),
    gui_testuj_slovo, NULL,
    NULL, 0, 0, 'p', GDK_CONTROL_MASK
  },
  {
    GNOME_APP_UI_ITEM, _("Vznam slova"),
    _("Vysvtl vznam zadanho slova."),
    gui_vyznamy_zobraz_okno, NULL,
    NULL, 0, 0, 'v', GDK_CONTROL_MASK
  },
  
  GNOMEUIINFO_SUBTREE(N_("rove hry potae"), obtiznost_submenu),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_NONE( _("Zakzan slova"),
    _("Prce se seznamem zakzanch slov."),
    gui_zakazana_zobraz_dialog),
  GNOMEUIINFO_ITEM_NONE( _("Logovn"), 
	  _("Budou se ukldat informace o prbhu hry do souboru"),
	   gui_logovani_zobraz_dialog),
  GNOMEUIINFO_TOGGLEITEM( _("Povolit hledn slov"),
	  _("Zake/povol pouit tlatka 'Najdi slova'."),
	  gui_ovladace_cheat_callback, NULL),
  GNOMEUIINFO_END
};

/* Menu Nastaven */
LOCAL GnomeUIInfo settings_menu[] = {

  GNOMEUIINFO_MENU_PREFERENCES_ITEM(zobraz_nastaveni, NULL),
  GNOMEUIINFO_END
};

/* Ve zabalme*/
LOCAL GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_MENU_GAME_TREE(game_menu),
  GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
  GNOMEUIINFO_SUBTREE(N_("N_stroje"), tools_menu),
  GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
  GNOMEUIINFO_MENU_HELP_TREE(help_menu),
  GNOMEUIINFO_END
};

/* -------------------- DEKLARACE GLOBLNCH FUNKC ------------------------- */

/* Pipoj 'main_menu' k aplikaci */
GLOBAL void menu_pripoj_menu(GtkWidget *app) {

   /* Pidej jednotliv menu k hlavnmu oknu */
   gnome_app_create_menus(GNOME_APP(app), main_menu);
     
}/* end of menu_pripoj_menu() */

/* -------------------------------------------------------------------------- */

/* Pipoj npovdu 'hints' ke status baru aplikace */
GLOBAL void menu_pripoj_hinty(GtkWidget *appbar) {

    gnome_app_install_appbar_menu_hints(GNOME_APPBAR(appbar), main_menu);
    
}/* end of menu_pripoj_hinty() */

/* -------------------------------------------------------------------------- */

/* END_OF_FILE menu.c */
