#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<gnome.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "zakazana_seznam.h"
#include "gui_okno.h"
#include "gui_deska.h"
#include "util.h"

/* --------------------- DEFINICE GLOBALNCH PROMNNCH --------------------- */

/* --------------------- DEFINICE LOCALNCH PROMNNCH ---------------------- */

LOCAL GtkWidget *zakaz_clist = NULL;
LOCAL boolean zakaz_vybrano = FALSE;     /* TRUE = vybrno slovo */
LOCAL int zakaz_row;          		/* Vybran dek v zakaz_clistu */

/* --------------- PLN FUNKN PROTOTYPY GLOBALNCH FUNKC ---------------- */

GLOBAL void gui_zakazana_zobraz_dialog(void);

/* ---------------- PLN FUNKN PROTOTYPY LOCALNCH FUNKC ---------------- */
LOCAL gint f_clist_porovnej(GtkCList *clist, 
	                            gconstpointer ptr1, gconstpointer ptr2);
LOCAL void zakaz_clist_callback(GtkWidget *w, gint row, gint column,
	                        GdkEventButton *event, gpointer data);

LOCAL void gui_zakazana_pridej_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_odeber_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_vymaz_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_nacti_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_uloz_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_info_callback(GtkWidget *w, gpointer data);
LOCAL void gui_zakazana_zobraz_seznam(void);
LOCAL void gui_zakazana_checkbox_callback(gpointer cbox);

/* -------------------------------------------------------------------------- */
/*                        DEFINICE JEDNOTLIVCH FUNKC                        */
/* -------------------------------------------------------------------------- */
LOCAL gint f_clist_porovnej(GtkCList *clist, 
                            gconstpointer ptr1, gconstpointer ptr2) {

    char *text1 = NULL;
    char *text2 = NULL;

    GtkCListRow *a = (GtkCListRow *) ptr1;
    GtkCListRow *b = (GtkCListRow *) ptr2;

    text1 = GTK_CELL_TEXT(a->cell[0])->text;
    text2 = GTK_CELL_TEXT(b->cell[0])->text;

    return( strcmp(text1, text2) );

}/* end of f_clist_porovnej() */

/* -------------------------------------------------------------------------- */

/* Reaguje na checkbox v okn  */
LOCAL void gui_zakazana_checkbox_callback(gpointer cbox) {

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cbox)) == TRUE)
	pouzit_zakazana = TRUE;
    else
	pouzit_zakazana = FALSE;

}/* end of gui_zakazana_checkbox_callback() */

/* -------------------------------------------------------------------------- */

/* Umon/znemon  odebrn/pravu dku v clistu */
LOCAL void zakaz_clist_callback(GtkWidget *w, gint row, gint column,
	                        GdkEventButton *event, gpointer data) {

    /* Unselect dku - znemon vybrn */
    if ((int) data == 1)
        zakaz_vybrano = FALSE;
    else {
        zakaz_vybrano = TRUE;
        zakaz_row = row;
    }

}/* end of zakaz_clist_callback() */

/* -------------------------------------------------------------------------- */

/* Pid do bin. stromu slovo, pokud tam u je, tak neprovede nic */
LOCAL void gui_zakazana_pridej_callback(GtkWidget *w, gpointer data) {

  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *textentry;
  int reply;
  gchar slovo[MAX_ROZMER+1];
  char *radek[1];
  
  dialog = gnome_dialog_new(N_("Zadejte zakzan slovo."), 
                            GNOME_STOCK_BUTTON_OK,
                            GNOME_STOCK_BUTTON_CANCEL,
                            NULL);
  gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
  gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
  gnome_dialog_set_default(GNOME_DIALOG(dialog), GNOME_OK);
  gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(data));
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  label = gtk_label_new("Vlote zakzan slovo.");
  textentry = gtk_entry_new_with_max_length(MAX_ROZMER);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                     label, TRUE, TRUE, GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                     textentry, TRUE, TRUE, GNOME_PAD);
  gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(textentry));
  gtk_widget_grab_focus(textentry);
  

  gtk_widget_show_all(dialog);

  reply = gnome_dialog_run(GNOME_DIALOG(dialog));
  
  if (reply == GNOME_OK) {      
      
      strcpy(slovo, gtk_entry_get_text(GTK_ENTRY(textentry)));

      /* Je-li to przdn dek */
      if (strcmp(slovo, "") == 0)
          return;
      
      /* Mezera nen taky povolena */
      if ( (strchr(slovo, (int) ' ')) != NULL ) {
          show_warning_window(GTK_WINDOW(data), 
    	      "Slovo nesm obsahovat mezeru!");
          return;
      }

      if ( (zakazana_seznam_existuje(slovo)) == TRUE) {
          show_info_window(GTK_WINDOW(data), "Slovo ji v seznamu existuje!");
	  return;
      }
      else {
	  zakazana_seznam_pridej(slovo);
	  radek[0] = slovo;
	  gtk_clist_append(GTK_CLIST(zakaz_clist), radek);
      }
  }/* end if */

#if GUI_ZAKAZANA_DEBUG
  zakazana_seznam_vypis();
#endif
    
  gtk_widget_destroy(dialog);   

}/* end of gui_zakazana_pridej_callback() */

/* -------------------------------------------------------------------------- */

/* Odeber z clistu dek se slovem a to odstran ze seznamu zakzanch
 * slov.
*/
LOCAL void gui_zakazana_odeber_callback(GtkWidget *w, gpointer data) {

    char *s = NULL;

    if (zakaz_vybrano == FALSE) {
        show_warning_window(GTK_WINDOW(data), "Nic nen vybrno!");
        return;
    }
    
    /* Zskej slovo */
    gtk_clist_get_text(GTK_CLIST(zakaz_clist), zakaz_row, 0, &s);

    /* Odeber ze seznamu */
    if ( (zakazana_seznam_odeber(s)) == FALSE ) {
        show_warning_window(GTK_WINDOW(data), 
		"Intern chyba!\n"
		"Slovo nen v seznamu!");
	return;
    }

    /* Odeber vizuln */
    gtk_clist_remove(GTK_CLIST(zakaz_clist), zakaz_row);
    
#if GUI_ZAKAZANA_DEBUG
    zakazana_seznam_vypis();
#endif

}/* end of gui_zakazana_odeber_callback() */

/* -------------------------------------------------------------------------- */

LOCAL void gui_zakazana_vymaz_callback(GtkWidget *w, gpointer data) {

    gtk_clist_clear(GTK_CLIST(zakaz_clist));
    zakazana_seznam_uvolni();
    zakaz_vybrano = FALSE;

}/* end of gui_zakazana_vymaz_callback() */

/* -------------------------------------------------------------------------- */

LOCAL void gui_zakazana_nacti_callback(GtkWidget *w, gpointer data) {

    gtk_clist_clear(GTK_CLIST(zakaz_clist));
    zakazana_seznam_uvolni();
    zakaz_vybrano = FALSE;
	
    /* Nati do seznamu */    
    if ( (zakazana_seznam_nacti()) == 4) {
        show_warning_window(GTK_WINDOW(data),
                         "Soubor se zakzanmi slovy se\n"
                         "nepodailo otevt!");
	return;
    }
    
    /* Napl oknko */
    gui_zakazana_zobraz_seznam();
   
}/* end of gui_zakazana_nacti_callback() */

/* -------------------------------------------------------------------------- */

/* Zobraz oknko s cestou k souboru se seznamem zakzanch slov. */
LOCAL void gui_zakazana_info_callback(GtkWidget *w, gpointer data) {

    char *s;
    
    s = g_strconcat("Cesta k souboru se zakzanmi slovy:\n",
	    "'", ZAKAZANA_SOUBOR, "'", NULL);
    
    show_info_window(GTK_WINDOW(data), s);

    g_free((gpointer) s);

}/* gui_zakazana_info_callback() */

/* -------------------------------------------------------------------------- */

/* Ulo aktuln seznam zakzanch slov na disk */
LOCAL void gui_zakazana_uloz_callback(GtkWidget *w, gpointer data) {

    boolean b;

    /* Nem cenu vytvet przdn soubor */
    if (zakazana_seznam_dej_delku() == 0)
	return;

    b = zakazana_seznam_uloz();

    if (b == TRUE)
	show_info_window(GTK_WINDOW(data), "Seznam uloen na disk.");
    else
        show_warning_window(GTK_WINDOW(data),
                            "Soubor se zakzanmi slovy se\n"
                            "nepodailo otevt pro zpis");

    
}/* end of gui_zakazana_uloz_callback() */

/* -------------------------------------------------------------------------- */

/* Zobraz dialogov okno s monost pidn/ubrn/naten/uloen seznamu
 * zakzanch slov 
*/
GLOBAL void gui_zakazana_zobraz_dialog(void) {

    GtkWidget *label;
    GtkWidget *checkbox;
    GtkWidget *pridej_b;
    GtkWidget *odeber_b;
    GtkWidget *vymaz_b;
    GtkWidget *uloz_b;
    GtkWidget *nacti_b;
    GtkWidget *help_b;

    GtkWidget *dialog; 
    GtkWidget *dialog_hbox;
    GtkWidget *dialog_vbox;
    GtkWidget *scroll_okno;

    dialog = gnome_dialog_new("Seznam zakzanch slov", 
	                      GNOME_STOCK_BUTTON_OK, NULL);
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    label = gtk_label_new("Seznam zakzanch slov");
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), 
		       label, FALSE, FALSE, 0);

    /* hbox rozdl okno na 2 plky, v lev bude okno se seznamem
     * a v prav buttonky.
    */
    dialog_hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), 
		       dialog_hbox, FALSE, FALSE, 0);


    scroll_okno = gtk_scrolled_window_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(dialog_hbox), scroll_okno, FALSE, FALSE, 5);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_okno),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
    zakaz_clist = gtk_clist_new(1); 
    gtk_container_add(GTK_CONTAINER(scroll_okno), zakaz_clist);
    gtk_signal_connect(GTK_OBJECT(zakaz_clist), "select-row",
             GTK_SIGNAL_FUNC(zakaz_clist_callback), (gpointer) 0);
    gtk_signal_connect(GTK_OBJECT(zakaz_clist), "unselect-row",
             GTK_SIGNAL_FUNC(zakaz_clist_callback), (gpointer) 1);
    gtk_clist_set_shadow_type(GTK_CLIST(zakaz_clist), GTK_SHADOW_ETCHED_OUT);
    gtk_clist_set_column_width(GTK_CLIST(zakaz_clist), 0, 130);
    gtk_clist_set_sort_column(GTK_CLIST(zakaz_clist), 0);
    gtk_clist_set_compare_func(GTK_CLIST(zakaz_clist), 
                              (GtkCListCompareFunc) f_clist_porovnej);
    gtk_clist_set_auto_sort(GTK_CLIST(zakaz_clist), TRUE);
    gtk_widget_set_usize(zakaz_clist, 150, 300);

    /* Uka aktuln stav */
    gui_zakazana_zobraz_seznam();

    /* prav strana dialogu, sada buttonk */
    dialog_vbox = gtk_vbox_new(FALSE, 0); 
    gtk_box_pack_start(GTK_BOX(dialog_hbox), dialog_vbox, FALSE, FALSE, 0);
    

    pridej_b = gtk_button_new_with_label("Pidej slovo");
    set_tooltip(pridej_b, "Pid tvar do seznamu zakzanch slov.");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), pridej_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(pridej_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_pridej_callback), 
			(gpointer) dialog);
    odeber_b = gtk_button_new_with_label("Odeber slovo");
    set_tooltip(odeber_b, "Odebere tvar ze seznamu zakzanch slov.");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), odeber_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(odeber_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_odeber_callback), 
			(gpointer) dialog);
    vymaz_b = gtk_button_new_with_label("Vyma seznam");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), vymaz_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(vymaz_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_vymaz_callback), 
			(gpointer) dialog);
    nacti_b = gtk_button_new_with_label("Nati seznam");
    set_tooltip(nacti_b, "Nate seznam z implicitnho souboru.");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), nacti_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(nacti_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_nacti_callback), 
			(gpointer) dialog);
    uloz_b = gtk_button_new_with_label("Ulo seznam");
    set_tooltip(uloz_b, "Ulo seznam do implicitnho souboru.");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), uloz_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(uloz_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_uloz_callback), 
			(gpointer) dialog);
    help_b = gtk_button_new_with_label("Info o seznamu");
    set_tooltip(help_b, "Zobraz cestu k souboru se seznamem.");
    gtk_box_pack_start(GTK_BOX(dialog_vbox), help_b, FALSE, FALSE, 0);
    gtk_signal_connect (GTK_OBJECT(help_b), "clicked",
                        GTK_SIGNAL_FUNC(gui_zakazana_info_callback), 
			(gpointer) dialog);

    checkbox = gtk_check_button_new_with_label("Pouvat zakzan slova");
    gtk_signal_connect (GTK_OBJECT(checkbox), "toggled",
                        GTK_SIGNAL_FUNC(gui_zakazana_checkbox_callback), 
			(gpointer) checkbox);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), checkbox,
                        FALSE, FALSE, 0);

    /* Zakrtni checkbox podle volby 'pouzit_zakazana' */
    gtk_widget_set_sensitive(GTK_WIDGET(checkbox), FALSE);
    if (pouzit_zakazana == TRUE)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
    else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), FALSE);

    gtk_widget_set_sensitive(GTK_WIDGET(checkbox), TRUE);
    
    gtk_widget_show_all(dialog);

    gnome_dialog_run_and_close(GNOME_DIALOG(dialog));

    /* Ulo aktuln nastaven checkboxu */
    gnome_config_push_prefix(GuiPrefix);
    gnome_config_set_int("POUZIT_ZAKAZANA", pouzit_zakazana);
    gnome_config_pop_prefix();
    gnome_config_sync();
    
}/* end of gui_zakazana_zobraz_dialog() */

/* -------------------------------------------------------------------------- */

LOCAL void gui_zakazana_zobraz_seznam(void) {
   
    int i;
    char *s;
    char *radek[1];
    
    /* Zobraz do oknka */
    for(i=0;i<zakazana_seznam_dej_delku();i++) {
	s = zakazana_seznam_dej_polozku(i);
	radek[0] = s;
	gtk_clist_append(GTK_CLIST(zakaz_clist), radek);
    }/* end for (i) */

}/* end of gui_zakazana_zobraz_seznam() */

/* -------------------------------------------------------------------------- */

/* END_OF_FILE gui_zakazana.c */
