#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<glib.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "util.h"

LOCAL GSList *externi_slovniky = NULL;

/* -------------------------------------------------------------------------- */

GLOBAL boolean slovniky_cesta_je_platna(char *cesta);
GLOBAL char *slovniky_dej_slovnik(int poradi);
GLOBAL char *slovniky_ziskej_popis(char *cesta_inf);
GLOBAL int slovniky_dej_pocet(void);
GLOBAL void slovniky_uvolni_seznam(void);
GLOBAL void slovniky_pridej_slovnik(char *cesta);
GLOBAL void slovniky_vypis_seznam(void);
GLOBAL void slovniky_odeber_slovnik(int poradi);

/* -------------------------------------------------------------------------- */

/* Inicializuje seznam absolutnch cest ke slovnkm */
GLOBAL void slovniky_uvolni_seznam(void) {

    if (externi_slovniky)
         g_slist_free(externi_slovniky);
    
    externi_slovniky = NULL;

}/* end of slovniky_uvolni_seznam() */

/* -------------------------------------------------------------------------- */

/* Pid cestu do seznamu slovnk */
GLOBAL void slovniky_pridej_slovnik(char *cesta) {

    char *path;

    if ( (path = (char *) malloc ((DMAX+1) * sizeof(char))) == NULL ) {
	 fprintf(stderr, "FATAL (slovniky.c) - malloc == NULL!\n");
	 exit(EXIT_FAILURE);
    }

    strcpy(path, cesta);

    externi_slovniky = g_slist_append(externi_slovniky, (gpointer) path);

}/* end of slovniky_pridej_slovnik() */

/* -------------------------------------------------------------------------- */

/* Odebere cestu ke slovnku */
GLOBAL void slovniky_odeber_slovnik(int poradi) {

    gpointer ktery;

    ktery = g_slist_nth_data(externi_slovniky, poradi);

    externi_slovniky = g_slist_remove(externi_slovniky, (gpointer) ktery);

#ifdef SLOVNIKY_DEBUG
    slovniky_vypis_seznam();
#endif
    
}/* end of slovniky_odeber_slovnik() */

/* -------------------------------------------------------------------------- */

/* Vrt poet externch slovnk */
GLOBAL int slovniky_dej_pocet(void) {

    return(g_slist_length(externi_slovniky));    

}/* end of slovniky_dej_pocet() */

/* -------------------------------------------------------------------------- */

/* Vrt cestu ke slovnk vloeno do seznamu v poad 'poradi'. */
GLOBAL char *slovniky_dej_slovnik(int poradi) {

    char *cesta;
    
    cesta = (char *) g_slist_nth_data(externi_slovniky, poradi);

    return(cesta);
    
}/* end of slovniky_dej_slovnik() */

/* -------------------------------------------------------------------------- */

/* Vype vechny cesty v seznamu slovnk */
GLOBAL void slovniky_vypis_seznam(void) {

    int i;
    char *cesta;

    printf("--SEZNAM SLOVNK ---\n");
    for(i=0;i<slovniky_dej_pocet();i++) {

	cesta = (char *) slovniky_dej_slovnik(i);
	
	printf("%s\n", cesta);
	
    }/* end for */
	
    printf("--KONEC VPISU SLOVNK ---\n");
	
}/* end of slovniky_vypis_seznam() */

/* -------------------------------------------------------------------------- */

/* Otestuje cestu, zda obsahuje pponu .mat a vrt TRUE/FALSE */
GLOBAL boolean slovniky_cesta_je_platna(char *cesta) {

    return( util_soubor_ma_priponu(cesta, ".mat") );

}/* end of slovniky_cesta_je_platna() */

/* -------------------------------------------------------------------------- */

/* Vrt prvn dek jako (char *) z danho souboru */
GLOBAL char *slovniky_ziskej_popis(char *cesta_inf) {

   char *s;

   FILE *soubor;

    if ((soubor = fopen(cesta_inf, "r")) == NULL) {
       fprintf(stderr,
       "Upozornn: (slovniky.c): soubor '%s' nenalezen!\n", cesta_inf);
       return(NULL);

    }/* end if */

    s = (char *) malloc(DMAX * sizeof(char));

    /* Soubor me bt przdn */
    if ((fgets(s, DMAX-2, soubor)) == NULL)
        return(NULL);

    /* Osekej o konec dku */
    if (s[strlen(s)-1] == '\r' || s[strlen(s)-1] == '\n')
        s[strlen(s)-1] = '\0';

    fclose(soubor);

    return(s);

}/* end of slovniky_ziskej_popis() */

/* -------------------------------------------------------------------------- */

/* END_OF_FILE slovniky.c */
