/* Program pro pevod slovnku na minimalizovan automat. 
 * Slovnk mus bt ve tvaru 'co dek, to slovo' (vertikl) a
 * mus bt setdn. Tdn NESM bt provedeno pro etinu,
 * ale podle ascii hodnoty (tzn. nap. "lampa, lska, laka"
 * je tzeno jako "lampa, laka, lska".
 *
 * Pedpokld se kdovn ISO8859-2.
 *
 * Autoi programu:
 * MG Ciura, S Deorowicz, "How to squeeze a lexicon".
 * Ukzkov program k textu.
 * 2001-05-04, 2001-09-13
 * 
*/

#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <locale.h>

/* -------------------------------------------------------------------------- */
			    /* Definice konstant */
/* -------------------------------------------------------------------------- */

#define MAX_DELKA_RETEZCE	60		/* pro Scrabble sta 15 */
#define MAX_VELIKOST_ABECEDY	256		/* monost a 256 vtv */ 
#define MAX_POCET_PRECHODU	1048576
#define MAX_POCET_STAVU         524288

#define VELIKOST_HASH_TABULKY	1048576		/* celkov poet poloek */
#define VELIKOST_HASH_POLOZKY	1024            /* velikost koliznho seznamu */

/* -------------------------------------------------------------------------- */
		   /* Definice prechodu - 32bitov hodnota */
/* -------------------------------------------------------------------------- */

typedef union {
    unsigned int cis_hodnota;       /* = 32b slo */
    struct {
	unsigned int posledni :  1; /*  1 bit  - posledni pechod ze stavu*/
	unsigned int do_stavu : 22; /* 22 bit - stavy */
	unsigned int znak     :  8; /*  8 bit - znak */
        unsigned int terminal :  1; /*  1 bit  - koncov pechod */
				    /* -------------- */
			            /* 32 bit celkem */ 			    
    } t;
} prechod;

/* Union zajist, e pi pstupu k pechodu vidm buto jednotliv poloky
 * nebo jednu selnou hodnotu z nich tvoenou
*/

/* -------------------------------------------------------------------------- */
		    /* Definice poloky haovac tabulky */
/* -------------------------------------------------------------------------- */


typedef struct hpolozka
{
    unsigned int adresa;	/* ukazatel na pozici stavu v automatu */
    unsigned int velikost;	/* jeho velikost */
    struct hpolozka *naslednik; /* nslednk v koliznm seznamu */
} polozka;


/* -------------------------------------------------------------------------- */
			    /* Globln promnn */
/* -------------------------------------------------------------------------- */


FILE *vertik_soubor;			/* vstupn soubor s etzci */
FILE *automat_soubor;			/* vstupn soubor s automatem */

unsigned long pocet_retezcu = 0;	/* natench z celho vstupu */
unsigned long pocet_znaku = 0;
unsigned long pocet_stavu = 0;

/* Automat bude reprezentovn polem pol, kde kad takov subpole
 * je stav, jeho prvky jsou jednotliv pechody 
*/

prechod automat[MAX_POCET_PRECHODU];	/* hlavn struktura - pole pechod */
unsigned int velikost_automatu;		/* celkov poet pechod */
unsigned int pocatecni_stav;            /* jeho pozice v automatu */

prechod  pom_stav[MAX_DELKA_RETEZCE+1][MAX_VELIKOST_ABECEDY];
unsigned int d_pom_stav[MAX_DELKA_RETEZCE+1];   /* poet pechod v 1 stavu */
int je_terminal[MAX_DELKA_RETEZCE+1];   	/* je pechod koncov? */

unsigned char pom_retezce[MAX_DELKA_RETEZCE+1]; /* etzce pro vpis */

polozka *hash_tabulka[VELIKOST_HASH_TABULKY];	/* haovac tabulka */
polozka *hash_polozka[MAX_POCET_PRECHODU/VELIKOST_HASH_POLOZKY];
int ht_nasl_polozka, ht_posl_pozice;

/* -------------------------------------------------------------------------- */
			    /* Procedury a funkce */
/* -------------------------------------------------------------------------- */

/* Pi chyb zahla o co jde a skoni */
void vypis_chybu(char *hlaska)
{
        fprintf(stderr, "%s\n", hlaska);
	fprintf(stderr, "--konme...\n");
	exit(EXIT_FAILURE);
}

/* -------------------------------------------------------------------------- */

unsigned int vypocti_hash_pozici(prechod *stav, unsigned int delka_stavu)
{
    unsigned int soucet = 0;
    unsigned int vysledek = 0;
    int i;

    /* postej vechny seln hodnoty pechod stavu */
    for (i=delka_stavu-1; i>=0; i--)
	soucet = soucet + stav[i].cis_hodnota;

    vysledek = ((soucet * 321087) >> 13) % VELIKOST_HASH_TABULKY; /* 324 027 */
    
    return(vysledek);
    
}/* end vypocti_hash_pozici () */


/* -------------------------------------------------------------------------- */


/* nacti automat ze vstupnho souboru */
void nacti_automat(char *nazev_souboru)
{

    if ((automat_soubor = fopen(nazev_souboru, "rb")) == NULL)
	vypis_chybu("Nemohu otevt vstupn soubor s vertiklem!");

    velikost_automatu = fread(automat, sizeof(automat[0]), MAX_POCET_PRECHODU, automat_soubor);
    fclose(automat_soubor);

    
    if (velikost_automatu >= 2) {
        /* vytvo pomocn stav ukazujc na poten stav */
	memcpy(&pocatecni_stav, &automat[0], sizeof(automat[0]));
	automat[0].t.do_stavu = automat[0].cis_hodnota;

	if (pocatecni_stav < velikost_automatu)
	    return;	/* OK, vstup naten spn */
	
    }/* end if */
    
    vypis_chybu("Chyba ve vstupnm souboru");
    
}/* end nacti_automat() */
 

/* -------------------------------------------------------------------------- */

/* ulo automat do souboru */
void uloz_automat(char *nazev_souboru)
{

    if ((automat_soubor = fopen(nazev_souboru, "wb")) == NULL)
	vypis_chybu("Nemohu otevt soubor pro zpis automatu!");

    /* vytvo pomocn stav ukazujc na pozici potenho stavu */
    automat[0].cis_hodnota = pocatecni_stav;
    
    if (fwrite(automat, sizeof(automat[0]), velikost_automatu, automat_soubor) 
        < velikost_automatu) vypis_chybu("Chyba pi zpisu automatu na disk!");
    
    fclose(automat_soubor);

}/* end uloz_automat() */

/* -------------------------------------------------------------------------- */

/* otevi/uloz soubor s etzci (vertikl) */
void zpracuj_vertikal(char *nazev_souboru, char *pristup)
{

    if ((vertik_soubor = fopen(nazev_souboru, pristup)) == NULL)
	vypis_chybu("Nemohu otevt vertikl!");

    if (setvbuf(vertik_soubor, NULL, _IOFBF, 8192) != 0)
        vypis_chybu("Chyba pi nastaven blokovho bufferovn!");
   
}/* end zpracuj_vertikal() */


/* -------------------------------------------------------------------------- */

/* Nate etzec ze vstupnho souboru a vrt jeho dlku */
int nacti_retezec(unsigned char *retezec)
{
    int delka;
    char *ptr;

    ptr = fgets(retezec, MAX_DELKA_RETEZCE, vertik_soubor);
    
    if (ptr == NULL)		/* konec souboru */
	return(0);

    delka = strlen(retezec);
    retezec[delka-1] = '\0';	/* '\n' v etzci nechceme */
    delka = strlen(retezec);

    pocet_retezcu++;		/* celkov poet natench etzc */

    return(delka);		/* vra dlku natenho etzce */
			    
}/* end nacti_retezec() */


/* -------------------------------------------------------------------------- */

/* Najdi shodn stav, ten vra, jinak vytvo nov stav a vra jeho
 * pozici v automatu
*/

/* 'stav' je ukazatel na pole a obsahuje 'delka_stavu' pechod */
unsigned int vytvor_stav(prechod *stav, unsigned int delka_stavu)
{
    int i;
    unsigned int pozice;

    polozka *ptr;
    unsigned int hash_adresa;
		    
    /* vytvo przdn stav */
    if (delka_stavu == 0) {
	stav[delka_stavu++].cis_hodnota = 0;
    }
   
    stav[delka_stavu-1].t.posledni = 1;	/* pechoz pechod stavu je posledn */
    
    /* spotej pozici v haovac tabulce, kam chceme umstt odkaz na stav */
    hash_adresa = vypocti_hash_pozici(stav, delka_stavu);
	/* je-li kolize, najdi msto pro novou poloku */
        for (ptr = hash_tabulka[hash_adresa]; ptr; ptr = ptr->naslednik) {
            if(ptr->velikost == delka_stavu) {
                 for (i = delka_stavu-1; i >= 0; i--) 
		    /* zkontroluj, zda obsahuje shodn pechody */
	            if (automat[ptr->adresa+i].cis_hodnota != stav[i].cis_hodnota)
	                break;
                    if (i < 0)	/* vechny pechody se shoduj, vra jeho adr */
			return ptr->adresa; 
	    }/*end if */
	}/* end for */
		    
    if (velikost_automatu + delka_stavu >= MAX_POCET_PRECHODU)
	vypis_chybu("Automat pekroil maximln poet pechod!");

    /* pidej stav do automatu */
    for (i = delka_stavu-1; i >= 0; i--)
	automat[velikost_automatu+i] = stav[i];
   
    if (ht_nasl_polozka >= VELIKOST_HASH_POLOZKY) {
	ht_nasl_polozka = 0;
	
	if ((hash_polozka[++ht_posl_pozice] = 
	(polozka *) malloc(sizeof(polozka) * VELIKOST_HASH_POLOZKY)) 
	== NULL)
	    vypis_chybu("Nedostatek pamti pro vytvoen poloky hash tab. !");
	
   }/* end if */
   
    /* aktualizuj seznamy v haovac tabulce */ 
    ptr = &hash_polozka[ht_posl_pozice][ht_nasl_polozka++];
    ptr->adresa= velikost_automatu; /* nov stav na pozici velikost automatu */
    ptr->velikost = delka_stavu;    /* bude obsahovat 'delka_stavu' pechod */
    ptr->naslednik = hash_tabulka[hash_adresa];/*spoj pedchdce a nslednka */
    hash_tabulka[hash_adresa] = ptr;

    
    pozice = velikost_automatu;
    velikost_automatu += delka_stavu;

    pocet_stavu++;
    return(pozice);
   
    /* pole pechod 'automat' vypad takto: 
     * [ [1.stav - pechody]  == pozice_stavu[0]
     *   [2.stav - pechody]  == pozice_stavu[1]
     *   [3.stav - pechody]  == pozice_stavu[2]
     *   ...                     ...
     * ]  
     *
    */
    
}/* end vytvor_stav() */



/* -------------------------------------------------------------------------- */

/* Vytvo automat podle algoritmu pro konstrukci Mealyho stroje */
void vytvor_automat(void)
{
    unsigned char s0[MAX_DELKA_RETEZCE+1] = "";
    unsigned char s1[MAX_DELKA_RETEZCE+1];
    unsigned int  i = 0, p, q;
    int k;

    prechod pom_prechod;
		
    pom_prechod.t.posledni = 0;		/* inicializace pechodu */

    velikost_automatu = 0;
   
    /* inicializuj haovac tabulku */
    for (k = VELIKOST_HASH_TABULKY-1; k >= 0; k--) 
        hash_tabulka[k] = NULL;

    /* inicializuj i poloky haovac tabulky */
    for (k = MAX_POCET_PRECHODU/VELIKOST_HASH_POLOZKY - 1; k >= 0; k--)
	hash_polozka[i] = NULL;
    
    ht_posl_pozice  = -1;
    ht_nasl_polozka = VELIKOST_HASH_POLOZKY;

    /* inicializace dlky stav */ 
    for (k=0; k<MAX_DELKA_RETEZCE+1; k++)
	d_pom_stav[k] = 0;


    /* natej etzce ze vstupu a utvej stavy a pechody */
    while ((q = nacti_retezec(s1)) != 0) {
       
   
        /* najdi spolen prefix */
        for (p=0; s1[p] == s0[p]; p++) ;

	if (s1[p] < s0[p]) 
           vypis_chybu("etzce ve vstupnm souboru nejsou seazeny!");
        
	/* utvo stavy pro suffix _pedchozho_ etzce */
	while (i > p) {
	    pom_prechod.t.do_stavu = vytvor_stav(pom_stav[i], d_pom_stav[i]);
	    i--;
	    pom_prechod.t.znak = s0[i];
	    pom_prechod.t.terminal = je_terminal[i+1];
	    pom_stav[i][d_pom_stav[i]++].cis_hodnota = pom_prechod.cis_hodnota;
	}/* i<= p <= q */
	   

	/* zkopruj suffix s1 do s0 */
	while (i < q) {
	    s0[i] = s1[i];
	    i++;
	    je_terminal[i] = 0;		/* i-t pechod stavu nen koncov */
	    d_pom_stav[i] = 0;		/* a i-t stav je te przdn */
	}/* i = q */

	s0[q] = '\0';			/* ukoni korektn etzec */
	je_terminal[q] = 1;         	/* posledn pechod stavu */

    }/* end while - natn vstupu */		    


    while (i > 0) {
        pom_prechod.t.do_stavu = vytvor_stav(pom_stav[i], d_pom_stav[i]);
	pom_prechod.t.terminal = je_terminal[i];
	pom_prechod.t.znak = s0[--i];
	pom_stav[i][d_pom_stav[i]++].cis_hodnota = pom_prechod.cis_hodnota;
    }/* end while */

    pocatecni_stav = vytvor_stav(pom_stav[0], d_pom_stav[0]);
    /* v pom_stav[0] jsou pechody z potenho stavu */
    automat[velikost_automatu].t.do_stavu = pocatecni_stav;
    /* posledn pechod ukazuje na poten stav */

    printf("nateno etzc:   %lu\n", pocet_retezcu);
    printf("celkem stav:      %lu\n", pocet_stavu);
    printf("celkem pechod:   %d\n", velikost_automatu);
    printf("velikost automatu: %d Byt\n\n", velikost_automatu*sizeof(unsigned));

}/* end vytvor_automat() */



/* -------------------------------------------------------------------------- */

/* Testuj pslunost etzce v automatu
 * Vrac: 0 - etzec neptomen
 *        1 - etzec nalezen
*/
int testuj_retezec(unsigned char *retezec)
{
    unsigned int pozice = 0;
    unsigned short int i;
    unsigned char w;


    for (i = 0; retezec[i]; i++) {
	
	/* zskej pozici nsledujcho stavu */
        pozice = automat[pozice].t.do_stavu;
        w = retezec[i];

        if (pozice > velikost_automatu)
	    vypis_chybu("Chyba v automatu!");
	
        /* nalezni pechod pod pslunm znakem */
        while (automat[pozice].t.znak != (unsigned int) w) {
	    
	    if (automat[pozice++].t.posledni)
		return 0;		/* z danho stavu nevede pechod */
	    
                if (pozice > velikost_automatu)
		    vypis_chybu("Chyba v automatu!");
		
        }/* end while */
	
    }/* end for */
    
    return(automat[pozice].t.terminal); /* nalezen pechod je koncov,
					   etzec je ptomen */

}/* end testuj_retezec() */



/* -------------------------------------------------------------------------- */

/* zobraz pouit programu */
void zobraz_pouziti(char *nazev)
{
    printf("Pouit: %s --vytvor <vertikl.txt> <automat.mat>\n", nazev);
    printf("         %s --vypis  <automat.mat> <vertikl.txt>\n", nazev);
    
    exit(EXIT_FAILURE);

}/*end zobraz_pouziti() */


/* -------------------------------------------------------------------------- */
			      /* Hlavn program */
/* -------------------------------------------------------------------------- */


int main(int argc, char *argv[])
{

    setlocale(LC_ALL, "cs_CZ.ISO8859-2"); 


    if (argc != 4) zobraz_pouziti(argv[0]);
    

    if ( (strcmp(argv[1], "--vytvor")) == 0) {
	printf("natm slovnk: '%s'\n\n", argv[2]);
	zpracuj_vertikal(argv[2], "r");
	vytvor_automat();
	printf("ukldm automat: '%s'\n\n", argv[3]);
	uloz_automat(argv[3]);
    } 
    else {
	zobraz_pouziti(argv[0]);
	fclose(vertik_soubor);
    }

    return(0);

}/* end main() */

/* END OF FILE */
