#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<glib.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "util.h"

/* ------------------------ DEFINICE LOCALNCH TYP ------------------------- */

/* --------------------- DEKLARACE GLOBALNCH PROMNNCH -------------------- */

/* --------------------- DEKLARACE LOCALNCH PROMNNCH --------------------- */

LOCAL GSList *zakaz_seznam = NULL;
LOCAL gint zakaz_seznam_delka;

/* --------------- PLN FUNKN PROTOTYPY GLOBALNCH FUNKC ---------------- */

GLOBAL boolean zakazana_seznam_existuje(char *slovo);
GLOBAL int zakazana_seznam_dej_delku(void);
GLOBAL char *zakazana_seznam_dej_polozku(int i);
GLOBAL void zakazana_seznam_uvolni(void);
GLOBAL void zakazana_seznam_vypis(void);
GLOBAL void zakazana_seznam_pridej(char *slovo);
GLOBAL int zakazana_seznam_odeber(char *slovo);
GLOBAL int zakazana_seznam_nacti(void);
GLOBAL int zakazana_seznam_uloz(void);

/* ---------------- PLN FUNKN PROTOTYPY LOCALNCH FUNKC ---------------- */

LOCAL gint f_porovnej(gconstpointer a, gconstpointer b);

/* ----------------------- DEFINICE GLOBALNCH FUNKC ----------------------- */

/* Odebere slovo ze seznamu zakzanch slov */
GLOBAL int zakazana_seznam_odeber(char *slovo) {

    GSList *element = NULL;

    element = g_slist_find_custom(zakaz_seznam, (gpointer) slovo,
                            (GCompareFunc) f_porovnej);

    /* 'element' je to, co chceme odstranit */
    if (element == NULL)
	return(FALSE);

   
    /* normln remove() nefungovalo */ 
    zakaz_seznam = g_slist_remove_link(zakaz_seznam, element);
    zakaz_seznam_delka = g_slist_length(zakaz_seznam);

    /* nutno jej jet uvolnit */
    g_slist_free_1(element);

    return(TRUE);

}/* end of zakazana_seznam_odeber() */

/* -------------------------------------------------------------------------- */

/* Nate do seznamu slov obsah souboru ~/scarabeus.scz 
 * Vrac: TRUE - OK
 *           3 - Soubor neexistuje
 *           4 - Soubor nelze otevt
*/

GLOBAL int zakazana_seznam_nacti(void) {

    FILE *soubor;
    char s[DMAX];

    /* Vra se, pokud neexistuje soubor s neznmmi slovy */
    if ( (util_soubor_existuje(ZAKAZANA_SOUBOR)) == FALSE)
	return(3);

    if ((soubor = fopen(ZAKAZANA_SOUBOR, "r")) == NULL)
	return(4);

    /* Proisti seznam */
    zakazana_seznam_uvolni();

    while( (fscanf(soubor, "%s", s)) > 0) {
	
	if ( (zakazana_seznam_existuje(s)) == FALSE )
	    zakazana_seznam_pridej(s);
	
	
    }/* end while */

    fclose(soubor);
    
#if ZAKAZANA_DEBUG
    zakazana_seznam_vypis();
#endif

    return(TRUE);
	
}/* end of zakazana_seznam_nacti() */

/* -------------------------------------------------------------------------- */

/* Ulo aktuln stav seznamu zakzanch slov na disk do souboru
 * ~/scarabeus.scz a vrac info o stavu OK/ERROR.
*/
GLOBAL int zakazana_seznam_uloz(void) {

    FILE *soubor;
    char *slovo;
    int i;

    /* Nelze soubor otevt? */
    if ((soubor = fopen(ZAKAZANA_SOUBOR, "w")) == NULL)
	return(FALSE);
    
    for(i=0;i<zakazana_seznam_dej_delku();i++) {
	slovo = zakazana_seznam_dej_polozku(i);
	fprintf(soubor, "%s\n", slovo);
    }/* end for (i) */
	
    fclose(soubor);

    return(TRUE);
    
}/* end of zakazana_seznam_uloz() */

/* -------------------------------------------------------------------------- */

/* Vrt TRUE, je-li slovo ji v seznamu zakzanch slov */
GLOBAL boolean zakazana_seznam_existuje(char *slovo) {

    GSList *l = NULL;

    l = g_slist_find_custom(zakaz_seznam, (gpointer) slovo, 
	                    (GCompareFunc) f_porovnej);

    if (l != NULL) {
	g_slist_free(l);
	return(TRUE);
    }

#if ZAKAZANA_DEBUG
    printf("Slovo '%s' v seznamu zakzanch nen!\n", slovo);
#endif

    return(FALSE);
    
}/* end of zakazana_seznam_existuje() */

/* -------------------------------------------------------------------------- */

/* Pid do seznamu tah dal poloku */
GLOBAL void zakazana_seznam_pridej(char *slovo) {
  
    char *pridej;

    /* Musme vytvoit novou poloku */
    if ( (pridej = (char *) malloc(MAX_ROZMER * sizeof(char))) == NULL ) {
	fprintf(stderr, "FATAL (zakazana_seznam.c) - malloc == NULL!\n");
	exit(EXIT_FAILURE);
    }

    strcpy(pridej, slovo);

    /* Prove pidn do seznamu */
    zakaz_seznam = g_slist_insert_sorted(
	     zakaz_seznam, (gpointer) pridej, (GCompareFunc) f_porovnej);

    zakaz_seznam_delka = g_slist_length(zakaz_seznam);
   
}/* end of zakazana_seznam_pridej() */

/* -------------------------------------------------------------------------- */

/* Vype seznam zakzanch slov na standardn vstup */
GLOBAL void zakazana_seznam_vypis(void) {

    char *s;
    int i;

    printf("---------------(%d)------------------\n", zakazana_seznam_dej_delku());
    for(i=0;i<zakazana_seznam_dej_delku();i++) {
	s = zakazana_seznam_dej_polozku(i);
	printf("%s\n", s);
    }
    printf("---------------------------------\n");
	
}/* end of zakazana_seznam_vypis() */

/* -------------------------------------------------------------------------- */

/* Uvoln pam po seznamu */
GLOBAL void zakazana_seznam_uvolni(void) {

    GSList *iterator;

    iterator = zakaz_seznam;

    /* Toto mus bt, jinak zahltme vechno! */
    while(iterator) {
        g_free(iterator->data);
        iterator = iterator->next;
    }

    if (zakaz_seznam)
	g_slist_free(zakaz_seznam);

    zakaz_seznam = NULL;

    zakaz_seznam_delka = 0;

}/* end of zakazana_seznam_uvolni() */

/* -------------------------------------------------------------------------- */

/* Pouzdc funkce pro zskn aktuln dlky seznamu */
GLOBAL int zakazana_seznam_dej_delku(void) {

    return( (int) zakaz_seznam_delka);

}/* end of zakaz_seznam_delka() */

/* -------------------------------------------------------------------------- */

/* Vra 'i'-tou poloku seznamu (slovo), pokud je 'i'
 * vt ne dlka seznamu, vra NULL.
*/
GLOBAL char *zakazana_seznam_dej_polozku(int i) {

    char *s;
   
    if (i > zakaz_seznam_delka) 
	return(NULL);

    s = (char *) g_slist_nth_data(zakaz_seznam, i);

    return(s);

}/* end of zakazana_seznam_dej_polozku() */

/* ---------------------- DEFINICE LOCALNCH FUNKC ------------------------- */

/* Porovnvac funkce pro g_slist_insert_sorted().
 * Vrac:  0 <=> a==b
 *        -1 <=> a > b
 *	  +1 <=> a < b
*/
LOCAL gint f_porovnej(gconstpointer a, gconstpointer b) {

    return(strcmp((gchar *) a, (gchar *) b));

}/* end of f_porovnej() */


/* -------------------------------------------------------------------------- */

/* END_OF_FILE zakazana_seznam.c */
